﻿unit uUploadPlugin;


interface


uses
 Windows ,
 Messages,
 SysUtils,
 StrUtils,
 Classes ,
 IniFiles,
 Math    ,

 BlckSock,
 SynSock ,
 SynAUtil,

 uLogger      ,
 uUploadAPI   ,
 uUploadConsts,
 uUploadUtils ;


type
 TUploadPlugin = class(TInterfacedObject, IUploadPlugin)
   private
    FLogger: TLogger;
    FSocket: TTCPBlockSocket;
    FMessageWnd: HWND;
    FCancelSocket: Boolean;

    procedure MessageWndMethod(var Message: TMessage);
    procedure SocketHeartbeat(Sender: TObject);
   public
    constructor Create;
    destructor Destroy; override;

    function Initialize(Reserved: IInterface; var ExtraInfo: WideString; out MessageWindow: HWND): BOOL; stdcall;
    function Finalize(Reserved: IInterface; var ExtraInfo: WideString): BOOL; stdcall;

    function GetParamI(Index: Integer): Int64; stdcall;
    function GetParamS(Index: Integer): WideString; stdcall;

    function UploadStream(Stream: ISerialStream; const FileName: WideString; var ExtraInfo: WideString; out URL: WideString): BOOL; stdcall;

    procedure ShowSettings(Reserved: IInterface); stdcall;
    procedure ShowAboutBox(Reserved: IInterface); stdcall;

    property Logger: TLogger read FLogger;
    property Socket: TTCPBlockSocket read FSocket;
  end;


implementation


{$R trueimages.ru.res}


const
 PluginName = 'trueimages.ru';
 Host       = 'trueimages.ru';
 Script     = 'http://trueimages.ru/upload.php';
 MainURL    = 'http://trueimages.ru/';




                    
//==================================================================================================
//  TUploadPlugin  
//==================================================================================================
 
                      
//==================================================================================================
constructor TUploadPlugin.Create; 
 begin
  inherited;
                  
  FLogger:=TLogger.Create
 end;

                       
//==================================================================================================
destructor TUploadPlugin.Destroy; 
 begin
  FreeAndNil(FLogger);

  inherited
 end;


//==================================================================================================
procedure TUploadPlugin.MessageWndMethod(var Message: TMessage);
 begin
  if Message.Msg=WM_UPLOAD_TERMINATE
   then   FCancelSocket:=True
   else   Message.Result:=DefWindowProc(FMessageWnd, Message.Msg, Message.WParam, Message.LParam)
 end;


//==================================================================================================
procedure TUploadPlugin.SocketHeartbeat(Sender: TObject);
 begin
  ProcessSocketHeartbeat(FSocket, FMessageWnd, FCancelSocket)
 end;


//==================================================================================================
function TUploadPlugin.Initialize(Reserved: IInterface; var ExtraInfo: WideString; out MessageWindow: HWND): BOOL;
 var
  Settings: TIniFile;
 begin
  //  Создание окна, которому будут посылаться события из главного приложения
  FMessageWnd:=AllocateHWnd(MessageWndMethod);
  MessageWindow:=FMessageWnd;
  FCancelSocket:=False;

  //  Загрузка настроек плагина
  if LoadPluginSettings(PluginName, ExtraInfo, Settings)
   then   Settings.Free;

  //  Логирование информации 
  if Logging
   then   begin
           Logger.AddLogRecord(rtInExtraInfo, ExtraInfo);
           ExtraInfo:='';
           Logger.AddLogRecord(rtOutExtraInfo, ExtraInfo)
          end
  else   ExtraInfo:='';

  Result:=True
 end;


//==================================================================================================
function TUploadPlugin.Finalize(Reserved: IInterface; var ExtraInfo: WideString): BOOL; stdcall;
 begin
  DeallocateHWnd(FMessageWnd);
             
  //  Логирование информации 
  if Logging
   then   begin
           Logger.AddLogRecord(rtInExtraInfo, ExtraInfo);
           ExtraInfo:='';
           Logger.AddLogRecord(rtOutExtraInfo, ExtraInfo)
          end
  else   ExtraInfo:='';

  Result:=True
 end;


//==================================================================================================
function TUploadPlugin.GetParamI(Index: Integer): Int64;
 begin
  case Index of
    PARAM_SUPPORT_IMAGES: Result:=1;
    PARAM_MAX_SIZE:       Result:=5*BytesPerMByte;
    PARAM_SUPPORT_PROXY:  Result:=1;
    else                  Result:=0;
   end
 end;


//==================================================================================================
function TUploadPlugin.GetParamS(Index: Integer): WideString;
 begin
  case Index of
    PARAM_NAME:         Result:=PluginName;
    PARAM_URL:          Result:=MainURL;
    PARAM_VERSION:      Result:='1.0.0';
    PARAM_COPYRIGHT:    Result:='';
    PARAM_AUTHOR:       Result:='Copyright © June 2010 by fd00ch';
    PARAM_SUPPORT_EXTS: Result:='.jpg;.jpeg;.gif;.png'
    else                Result:='' 
   end
 end;

                                   
//==================================================================================================
procedure TUploadPlugin.ShowSettings(Reserved: IInterface); stdcall;
 begin
  MessageBoxW(0, 'Данный плагин не имеет настроек', 'Настройка', MB_ICONINFORMATION or MB_OK)
 end;


//==================================================================================================
procedure TUploadPlugin.ShowAboutBox(Reserved: IInterface); stdcall;
 begin
  MessageBoxW(0,
              'Плагин для загрузки изображений на бесплатный хостинг trueimages.ru',
              'О плагине', 
              MB_ICONINFORMATION or MB_OK)
 end;  
 
                                                          
//==================================================================================================
function TUploadPlugin.UploadStream(Stream: ISerialStream; const FileName: WideString; var ExtraInfo: WideString; out URL: WideString): BOOL;
 var
  Ext   ,
  RawStr: string;


  function ReadRawStr: Boolean;
   begin
    RawStr:='';
    repeat
     RawStr:=RawStr+Socket.RecvPacket(TimeOut);
    until Socket.LastError<>0;
    Result:=Socket.LastError<>WSAETIMEDOUT;
    
    if not Result
     then   ExtraInfo:=NameError+StrErrorTimeout;

    if Logging
     then   Logger.AddLogRecord(rtRecieve, RawStr)
   end;


  function GetMimeType: string;
   begin
    if SameText(Ext, '.jpg') or SameText(Ext, '.jpeg')
     then   Result:='image/jpeg'
     else   if SameText(Ext, '.gif')
             then   Result:='image/gif'
             else   if SameText(Ext, '.png')
                     then   Result:='image/png'
                     else   Result:=''
   end;


 var
  Headers,
  Start  ,
  Finish ,
  Bound  ,
  Temp   : string;
 begin
  Result:=False;
  Ext:=ExtractFileExt(FileName);

  if Logging
   then   Logger.AddLogRecord(rtInExtraInfo, ExtraInfo);

  try
    //----------------------------------------------------------------------------------------------
    //  Если быстрый режим выключен - сначала проверяем доступность сервера  
    //----------------------------------------------------------------------------------------------
    if not FastMode
     then   if not CheckHostAvailability(FSocket, SocketHeartbeat, FCancelSocket, Host, DefaultPort, MainURL, ExtraInfo)
             then   Exit;



    //----------------------------------------------------------------------------------------------
    //  Основное подключение к серверу и отсылка данных
    //----------------------------------------------------------------------------------------------
    if not DoConnectSocket(FSocket, SocketHeartbeat, FCancelSocket, Host, DefaultPort, ExtraInfo)
     then   Exit;             

    Randomize;
    Random(MaxInt);
    Bound:=IntToHex(Random(MaxInt), 8);
  
    //  Составление начальных данных об изображении
    Start:='--'+Bound+CRLF+
           'Content-Disposition: form-data; name="resize"'+CRLF+CRLF+
           '0'+CRLF+

           '--'+Bound+CRLF+
           'Content-Disposition: form-data; name="quality"'+CRLF+CRLF+
           '85'+CRLF+

           '--'+Bound+CRLF+
           'Content-Disposition: form-data; name="MAX_FILE_SIZE"'+CRLF+CRLF+
           '500000000'+CRLF+

           '--'+Bound+CRLF+
           'Content-Disposition: form-data; name="fileToUpload[]"; filename="'+UTF8Encode(FileName)+'"'+CRLF+
           'Content-Type: '+GetMimeType+CRLF+CRLF;


    //  Составление конечных данных об изображении
    Finish:=CRLF+

            '--'+Bound+'--'+CRLF;

    //  Составление заголовков
    Headers:='POST '+Script+' HTTP/1.0'+CRLF+
             'Host: '+Host+CRLF+
             'User-Agent: '+UserAgent+CRLF+
             'Referer: '+MainURL+CRLF+
             GetProxyAuthHeader(ProxySettings)+
             'Content-Type: multipart/form-data; boundary='+Bound+CRLF+
             'Content-Length: '+IntToStr(Integer(Stream.Size)+Length(Start)+Length(Finish))+CRLF+CRLF;   

    if Logging
     then   Logger.AddLogRecord(rtSend, Headers+Start+FileName+' ['+FormatFileSize(Stream.Size)+']'+Finish);

    //  Посылка заголовков
    Socket.SendString(Headers);
    //  Посылка начальных данных об изображении
    Socket.SendString(Start);
    //  Посылка самого изображения
    if not SendStream(Socket, Stream, ExtraInfo)
     then   URL:='';
    //  Посылка конечных данных об изображении
    Socket.SendString(Finish);
    
    //  Приём данных от хоста
    if not ReadRawStr
     then   Exit;

    //  Вычленение из полученных данных информации об адресе загруженной картинки
    Result:=Pos('[IMG]', RawStr)>0;
    if Result
     then   begin
             //  Получение ссылки на превью
             Temp:=RawStr;
             Parse(Temp, '[IMG]');
             ExtraInfo:=NameImagePreview+Trim(Parse(Temp, '[/IMG]'));
             //  Получение ссылки на полноразмерное изображение
             Parse(Temp, '[IMG]');
             URL:=Trim(Parse(Temp, '[/IMG]'));
             //  Получение ссылки на страницу с файлом
             Temp:=RawStr;
             Parse(Temp, '[URL=&quot;');
             ExtraInfo:=ExtraInfo+CRLF+NameImagePage+Parse(Temp, '&quot;]')
            end
     else   ExtraInfo:=NameError+StrErrorGetLink
   finally
    if Logging and not Result
     then   begin
             Logger.AddLogRecord(rtURL, URL);
             Logger.AddLogRecord(rtOutExtraInfo, ExtraInfo);
             Logger.SaveToFile(PluginName)
            end;
    FreeAndNil(FSocket)
   end
 end;
 

end.
